PROGRAM copr3
  !// The module globvars contains all the global variables
  USE globvars
  !// The module methods contains all the subroutines we use in
  !// this program
  !// Note that the function a2f is in a separate file strconv.f90
  !// which also contains similiar functions for integers and
  !// double precision numbers
  USE methods

  IMPLICIT NONE
  
  !///////////// Front Matter ///////////////////////////////////////
  !// PURPOSE: 
  !// To solve Computer Problem #3 GEF4510
  !//
  !//----------------------------------------------------
  !// METHOD: 
  !// We are asked to solve the dimensionless advection equation:
  !//
  !//           dD/dt + dD/dx = 0,  for 0 < x < 1
  !//
  !// where D is the tracer concentration, u=1 is the dimensionless 
  !// advection speed, and x=1 the dimensionless width of the 
  !// computational domain. 
  !//
  !// To this end we are asked to use three methods, namely
  !//
  !//   (i) the leap-frog method
  !//  (ii) the upstream method, and
  !// (iii) the Lax-Wendroff method.
  !//
  !// In finite difference form these methods are:
  !//
  !//  (i)   D(j,n+1) = D(j,n-1) - COURANTNR*(D(j+1,n) - D(j-1,n))
  !//
  !//  (ii)  D(j,n+1) = D(j,n) - COURANTNR*(D(j+1,n) - D(j  ,n)) 
  !//        when CCOURANTNR < 0, and
  !//        D(j,n+1) = D(j,n) - COURANTNR*(D(j  ,n) - D(j-1,n)) 
  !//        when COURANTNR > 0.
  !//
  !//  (iii) D(j,n+1) = 0.5*(D(j+1,n) + D(j-1,n)) - C2*(D(j+1,n) - D(j-1,n))
  !//        D(j,n+2) = D(j,n) - COURANTNR*(D(j+1,n+1) - D(j-1,n+1))
  !//
  !// where COURANTNR = Dt/Dx and C2 = 0.5*C. Note that the Lax-Wendroff 
  !// method alternates between the two specified methods.  Thus we
  !// first solve for all points at time level n+1, then for all points
  !// at time level n+2. We also note that while the leapfrog and 
  !// Lax-Wendroff are three time level methods, the upstream method 
  !// is a two time level method 
  !//
  !// The initial condition is a Gaussian bell function given by
  !//
  !//       D(j,0) = exp(-((X -X0)/BELL_DIST)^2 
  !//
  !// where initial maximum dimensionless concentration is set to 1, 
  !// X0 is the position (dimensionless) of the maximum initial 
  !// concentration, and SW is a (dimensionless) measure of the 
  !// width of the bell shape.  We are asked to solve the equation in 
  !// dimensionless form in which the dimensionless variables are D=D/D0,
  !// x=x/LX, and u=u/U0.  The dimensionless time then becomes t=t/T, 
  !// where T=LX/U0.  let LX = 50 km, 
  !// D0 = 100%, U0 = 1 m/s (ocean application), and X0 = 0.5*LX. 
  !//
  !// We are asked to make at least two experiments in which SW varies. 
  !// One is BELL_DIST=0.1 (wide bell) and a second with BELL_DIST=0.001
  !// (narrow bell).
  !// The number of space increments is set to JMAX = 101, including the 
  !// boundary points at J=1 (x=0) and J = JMAX (x=1).
  !// 
  !//---------------------------------------------------------------------
  !// STATUS:
  !// Coded by: Lars Petter Red, met.no, based on an earlier version
  !// Date    : 15.10.2008
  !// 
  !//---------------------------------------------------------------------
  !// REVISION HISTORY:
  !// Revised by: Gunnar Wollan
  !// Date      : 10.10.2008
  !// Rewritten into Fortran 95 code using modules and incorporating
  !// command-line argument input method in addition to interactive input
  !//---------------------------------------------------------------------
  !//
  !// VARIABLES:
  !// JJ          : Maximum number of grid points along x-axis
  !//               NOT USED IN THIS VERSION
  !// NT          : Number of time levels necessary
  !//               NOT USED IN THIS VERSION
  !// NTIME       : Time step counter
  !// NTMAX       : Maximum number of time steps that will be performed
  !// X(JMAX)     : Value of x associated with grid point
  !// D(JMAX,NT)  : Concentration (or density)
  !// DP(JMAX,NP) : Tracer concentration for printing
  !//
  !// PHYSICAL CONSTANTS (SI units):
  !// DX          : Space increment along x-axis (in meters)
  !// DT          : Time step (in seconds)
  !// D0          : Maximum initial concentration (=100 %)
  !//               NOT USED IN THIS VERSION
  !// U0          : Advection speed (= 1 m^2/s)
  !//               NOT USED IN THIS VERSION
  !// BELL_DIST   : Width of Gaussian bell distribution (to be varied)
  !// X0          : Initial position of maximum initial concentration
  !// LX          : Length of computational domain
  !//               NOT USED IN THIS VERSION
  !//
  !// AUXILIARY CONSTANT (SI units):
  !//-------------------------------------------------------------------
  !// J           : Grid point counter
  !// COURANTNR   : Courant number (= U0*DT/DX)
  !//               FORMULA NOT USED IN THIS VERSION
  !// C2          : C2=0.5*COURANTNR
  !// ARG         : Auxiliary argument for bell function
  !// NP          : Number of fields to be plotted (=4 including initial)
  !// N1C         : Number of time levels for completing one cycle
  !// N1,N2,N3,N4 : Time steps for output
  !//               N1 NOT USED IN THIS VERSION
  !// METHOD      : Flag determining which method to use:
  !//               leapfrog, upstream, lax_wendroff 
  !// IPP         : Flag to determine if Euler step should be skipped
  !//               when using the leapfrog method (1=yes,2=no)
  !//               NOT USED IN THIS VERSION
  !// IP          : Flag determining initial distribution of the 
  !//               consentration D (1=sinus, 2=spike, 3=gaussian bell) 
  !//               NOT USED IN THIS VERSION
  !//
  !// ADDITIONAL VARIABLES
  !//-------------------------------------------------------------------
  !// argc        : Command line argument counter (contains the number of
  !//               arguments given on the command line in addition to
  !//               the program name)
  !// filename    : Contianing the filename of the result of the computing
  !// iargc       : External function returning the number of command
  !//               line arguments (stored in argc)
  !// a2f         : External function converting the ASCII representation
  !//               of a real number to the binary representation
  !//
  !/////////// END FRONT MATTER //////////////////////////////////////////

  !///////////////////////////////////////////////////////////////////////
  !//
  !// USAGE: cpor3 <method> <courant number> <gaussian bell distribution> 
  !//              <number of repeated cycles (3 values)>



  !// Variable declarations
  INTEGER                           :: argc, i
  CHARACTER(LEN=80)                 :: filename
  !// External declarations
  INTEGER, EXTERNAL                 :: iargc
  REAL, EXTERNAL                    :: a2f

  !// Get number of input arguments
  argc = iargc()
  !// Test if we have all we need
  IF (argc < 6) THEN
    !// No, call the user dialog
    CALL ui()
  ELSE
    !// Yes, let us get the input arguments
    !// Get the method
    CALL getarg(1,method)
    !// Loop and get the rest of the arguments
    DO i = 2, 6
      CALL getarg(i,argv(i-1))
    END DO
  END IF

  !// Initialize variables
  CALL init_vars()

  !// Check that we have chosen one of the
  !// available methods
  !// Do we have the leapfrog method?
  IF(TRIM(method) .EQ. 'leapfrog') THEN
    !// Yes, we fill in the filename into the filename
    !// variable
    filename = 'leapfrog_data.dat'
    !// Loop through the timesteps
    DO ntime = 1, ntmax
      !// Call the leapfrog subroutine
      CALL leapfrog()
      !// Save the chosen data
      CALL save_data(ntime)
      !// Swap the value of the timestep variables
      CALL swap_timesteps()
    END DO
    !// Write the result to the filename
    CALL write_data(filename)
  !// Do we have the upstream method?
  ELSE IF(TRIM(method) .EQ. 'upstream') THEN
    !// Yes, we fill in the filename into the filename
    !// variable
    filename = 'upstream_data.dat'
    !// Loop through the timesteps
    DO ntime = 1, ntmax
      CALL upstream()
      !// Save the chosen data
      CALL save_data(ntime)
      !// Swap the value of the timestep variables
      CALL swap_timesteps()
    END DO
    !// Write the result to the filename
    CALL write_data(filename)
  !// Do we have the lax_wendorff method?
  ELSE IF(TRIM(method) .EQ. 'lax_wendorff') THEN
    !// Yes, we fill in the filename into the filename
    !// variable
    filename = 'lax_wendorff_data.dat'
    !// Loop through the timesteps
    DO ntime = 1, ntmax
      CALL lax_wendroff()
      !// Save the chosen data
      CALL save_data(ntime)
      !// Swap the value of the timestep variables
      CALL swap_timesteps()
    END DO
    !// Write the result to the filename
    CALL write_data(filename)
  ELSE
    !// We have an unknown method so we stop the program
    PRINT *, 'Unknown method ', TRIM(method)
    PRINT *, 'Terminating program'
    STOP
  END IF
  PRINT *, 'Program terminated successfully!'
  PRINT *, 'The result is found in the file: ', TRIM(filename)

END PROGRAM copr3
