MODULE methods
  USE globvars
  IMPLICIT NONE

CONTAINS

  SUBROUTINE ui()
    IMPLICIT NONE
    !// User interface for interactive
    !// fetching of inital data
    !// Input:
    !//  <method>
    !//  <courant number> 
    !//  <gaussian bell distribution> 
    !//  <number of repeated cycles (3 values)>

    WRITE(*,*) 'The following methods are available:'
    WRITE(*,*) 'leapfrog'
    WRITE(*,*) 'upstream'
    WRITE(*,*) 'lax_wendorff'
    WRITE(*,FMT='(A)',ADVANCE='NO') 'Enter the method : '
    READ(*,*) method
    WRITE(*,FMT='(A)',ADVANCE='NO') 'Enter the courant number: '
    READ(*,*) courantnr
    WRITE(*,FMT='(A)',ADVANCE='NO') 'Enter the Gaussian bell distribution: '
    READ(*,*) bell_dist
    WRITE(*,FMT='(A)',ADVANCE='NO') 'Enter the number of cycles (3 values): '
    READ(*,*) cycles(1), cycles(2), cycles(3)
    interactive = .TRUE.
  END SUBROUTINE ui

  SUBROUTINE init_vars()
    IMPLICIT NONE
    !// Initialize global variables
    !// Local variables
    INTEGER                    :: ix, jx
    REAL, EXTERNAL             :: a2f
    IF(.NOT. interactive) THEN
      !// Get the courant number
      !// Convert from ascii representetion to
      !// binary representation
      courantnr = a2f(TRIM(argv(1)))
      !// Is the courant number large enough?
      IF(courantnr < 1.) THEN
        !// No, print an error message and stop program
        PRINT *, 'Courant number ', courantnr, ' is too small'
        STOP
      END IF
      !// Get the gaussian bell distribution
      bell_dist = a2f(TRIM(argv(2)))
      !// Get the first cycle number
      cycles(1) = a2f(TRIM(argv(3)))
      !// Get the second cycle number
      cycles(2) = a2f(TRIM(argv(4)))
      !// Get the third cycle number
      cycles(3) = a2f(TRIM(argv(5)))
    END IF


    !// Debug info
!    PRINT *, 'Start debug info'
!    PRINT *, 'Method: ', TRIM(method)
!    PRINT *, 'Courant number: ', courantnr
!    PRINT *, 'Gaussian bell: ', bell_dist
!    PRINT *, 'Cycles: ', cycles
!    PRINT *, 'End debug info'

    !// Calculate some constants
    !// Space increment
    dx = 1./(jmax-1)
    !// Courant number
    courantnr = 1./courantnr
    !// Time step
    dt = courantnr/dx
    !// Help variable
    c2 = 0.5 * courantnr
    !// Number of time levels to complete one cycle
    n1c = FLOAT((jmax-1))/courantnr
    !// First time level at which results are saved
    n2 = INT(n1c * cycles(1))
    !// Second time level at which results are saved
    n3 = INT(n1c * cycles(2))
    !// Third time level at which results are saved
    n4 =INT( n1c * cycles(3))

    !// Debug info
!    PRINT *, 'N4 ', n4
    !// Maximum number of time levels
    ntmax = n4 + 1

    !// Debug info
!    PRINT *, n1c, n2, n3, n4, cycles, ntmax

    !// Calculate the location of grid points on the x-axis
    DO ix = 1, jmax
      x(ix) = (ix - 1) * dx
    END DO
    x0 = 0.5
 
    !// First tracer concentration (Gaussian bell)
    DO jx = 1, ntl
      DO ix = 1, jmax
        arg = (x(ix) - x0)/bell_dist
        arg = -arg * arg
        d(ix,jx) = EXP(arg)
      END DO
    END DO

    !// Save initial concentrations distribution for later plotting
    DO ix = 1, jmax
      dp(ix,1) = d(ix,1)
    END DO

    !// Next time
    IF(TRIM(method) .EQ. 'upstream') THEN
      in = 1
      inew = 2
    ELSE
      iold = 1
      in = 2
      inew = 3
    END IF
  END SUBROUTINE init_vars

  SUBROUTINE save_data(ntime)
    IMPLICIT NONE
    !// Input argument
    INTEGER                    :: ntime
    !// Local variables
    INTEGER                    :: ix
    !// Should we save this timestep for later plotting?
    IF(ntime == n2) THEN
      DO ix = 1, jmax
        dp(ix,2) = d(ix,inew)
      END DO
    END IF
    IF(ntime == n3) THEN
      DO ix = 1, jmax
        dp(ix,3) = d(ix,inew)
      END DO
    END IF
    IF(ntime == n4) THEN
      DO ix = 1, jmax
        dp(ix,4) = d(ix,inew)
      END DO
    END IF
  END SUBROUTINE save_data

  SUBROUTINE swap_timesteps()
    IMPLICIT NONE
    !// Local variables
    INTEGER                    :: isave
    IF(TRIM(method) .EQ. 'upstream') THEN
      isave = in
      in = inew
      inew = isave
    ELSE
      isave = iold
      iold = in
      in = inew
      inew = isave
    END IF
  END SUBROUTINE swap_timesteps

  SUBROUTINE write_data(filename)
    IMPLICIT NONE
    !// Input argument
    CHARACTER(LEN=80)          :: filename
    !// Local variables
    INTEGER                    :: ix, rstat
    INTEGER, PARAMETER         :: lun = 10
    OPEN(UNIT=lun,FILE=filename,FORM='FORMATTED',IOSTAT=rstat)
    IF(rstat /= 0) THEN
      PRINT *, 'Error in opening file, status', rstat
      RETURN
    END IF
    !// Write values to the file
    DO ix = 1, jmax
      WRITE(UNIT=lun, FMT='(2X,5F10.4)',IOSTAT=rstat) x(ix), dp(ix,1), &
                                                      dp(ix,2), dp(ix,3), &
                                                      dp(ix,4)
    END DO
    !// Close the file
    CLOSE(UNIT=lun)
  END SUBROUTINE write_data

  SUBROUTINE leapfrog()
    IMPLICIT NONE
    !// Local variables
    INTEGER                    :: ix, jm1
    DO ix = 1, jmax - 1
      IF (ix == 1) THEN
        jm1 = jmax - 1
      ELSE
        jm1 = ix - 1
      END IF
      IF (ntime == 1) THEN
        d(ix,inew) = d(ix,in) -c2*(d(ix+1,in) - d(jm1,in))
      ELSE
        d(ix,inew) = d(ix,iold) - courantnr*(d(ix+1,in) - d(jm1,in))
      END IF
    END DO
    d(ix,inew) = d(1,inew)
    d(ix,in) = d(1,in)

  END SUBROUTINE leapfrog



  SUBROUTINE upstream()
    IMPLICIT NONE
    !// Local variables
    INTEGER                    :: ix, jm1
    DO ix = 1, jmax - 1
      IF (ix == 1) THEN
        jm1 = jmax - 1
      ELSE
        jm1 = ix - 1
      END IF
      IF (courantnr < 0.) THEN
        d(ix,inew) = d(ix,in) -courantnr*(d(ix+1,in) - d(ix,in))
      ELSE
        d(ix,inew) = d(ix,in) - courantnr*(d(ix,in) - d(jm1,in))
      END IF
    END DO
    d(ix,inew) = d(1,inew)

  END SUBROUTINE upstream


  SUBROUTINE lax_wendroff()
    IMPLICIT NONE
    !// Local variables
    INTEGER                    :: ix, jm1
    DO ix = 1, jmax - 1
      IF (ix == 1) THEN
        jm1 = jmax - 1
      ELSE
        jm1 = ix - 1
      END IF
      d(ix,inew) = 0.5*(d(ix+1,in) + d(jm1,in) - c2*(d(ix+1,in) - d(jm1,in)))
    END DO
    d(ix,inew) = d(1,inew)

  END SUBROUTINE lax_wendroff



END MODULE methods
